/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2013 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    FTP_Server.c
 * Purpose: FTP Server example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include "cmsis_os.h"                   /* CMSIS RTOS definitions             */
#include "rl_net.h"                     /* Network definitions                */
#include "rl_fs.h"                      /* File System definitions            */

#include "bsp.h"
#include "em_cmu.h"

static void BlinkLed (void const *arg);
osThreadDef(BlinkLed, osPriorityNormal, 1, 0);

/*----------------------------------------------------------------------------
  Thread 'BlinkLed': Blink the LEDs on an eval board
 *---------------------------------------------------------------------------*/
static void BlinkLed (void const *arg) {
  const uint8_t led_val[16] = { 0x48,0x88,0x84,0x44,0x42,0x22,0x21,0x11,
                                0x12,0x0A,0x0C,0x14,0x18,0x28,0x30,0x50 };
  int cnt = 0;

  while(1) {
    // Every 100 ms
    BSP_LedsSet (led_val[cnt]);
    if (++cnt >= sizeof(led_val)) {
      cnt = 0;
    }
    osDelay (100);
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {
  uint16_t reg;

  // HFXO (High frequency crystal oscillator) is 48MHz
  CMU_OscillatorEnable (cmuOsc_HFXO, true, true);
  CMU_ClockSelectSet   (cmuClock_HF,    cmuSelect_HFXO);
  CMU_ClockDivSet      (cmuClock_HFPER, cmuClkDiv_1);

  BSP_Init(BSP_INIT_DK_EBI);
  BSP_PeripheralAccess (BSP_ETH,     true);
  BSP_PeripheralAccess (BSP_MICROSD, true);
  BSP_LedsInit();

  // Reset external EMAC/PHY
  reg  = BSP_RegisterRead (&BC_REGISTER->CLKRST);
  reg |= (1 << BC_CLKRST_ETH_SHIFT);
  BSP_RegisterWrite (&BC_REGISTER->CLKRST, reg);
  osDelay(5);
  reg &= ~(1 << BC_CLKRST_ETH_SHIFT);
  BSP_RegisterWrite (&BC_REGISTER->CLKRST, reg);
  osDelay(5);

  net_initialize ();
  if (finit("") == fsOK) {
    fmount("");
  }

  osThreadCreate (osThread(BlinkLed), NULL);

  while(1) {
    net_main ();
    osThreadYield ();
  }
}
