/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::Network
 * Copyright (c) 2004-2013 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    SNMP_Agent.c
 * Purpose: SNMP Agent example
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "cmsis_os.h"                   /* CMSIS RTOS definitions            */
#include "rl_net.h"                     /* Network definitions               */
#include "rl_net_lib.h"                 /* Network library definitions       */

#include "bsp.h"
#include "em_cmu.h"

bool LCDupdate;
char lcd_text[2][16+1] = { " ", "Waiting for DHCP" };

extern LOCALM localm[];
#define MY_IP localm[NETIF_ETH].IpAdr

static void Display (void const *arg);

osThreadDef(Display, osPriorityNormal, 1, 0);

/// Read digital inputs
uint8_t get_button (void) {
  uint8_t val;

  val  = BSP_PushButtonsGet() & BC_UIF_PB_MASK;
  val |= (BSP_JoystickGet()   & BC_UIF_JOYSTICK_MASK) << 4;

  return (val);
}

/// Check DHCP if IP address assigned
static void dhcp_check (void) {
  static uint8_t ip[IP_ADDR_LEN];

  if (!memcmp (&MY_IP, ip, IP_ADDR_LEN)) {
    return;
  }
  // IP address has changed
  memcpy (ip, &MY_IP, IP_ADDR_LEN);
  sprintf((char *)lcd_text[0],"IP address:");
  sprintf((char *)lcd_text[1],"%d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);
  LCDupdate = true;
}

/*----------------------------------------------------------------------------
  Thread 'Display': LCD display handler
 *---------------------------------------------------------------------------*/
static void Display (void const *arg) {

  // LCD initialization here
  LCDupdate = true;

  while(1) {
    if (LCDupdate == true) {
      // Write the update on the LCD here
      LCDupdate = false;
    }
    osDelay (250);
  }
}

/*----------------------------------------------------------------------------
  Main Thread 'main': Run Network
 *---------------------------------------------------------------------------*/
int main (void) {
  uint16_t reg;

  // HFXO (High frequency crystal oscillator) is 48MHz
  CMU_OscillatorEnable (cmuOsc_HFXO, true, true);
  CMU_ClockSelectSet   (cmuClock_HF,    cmuSelect_HFXO);
  CMU_ClockDivSet      (cmuClock_HFPER, cmuClkDiv_1);

  BSP_Init(BSP_INIT_DK_EBI);
  BSP_PeripheralAccess (BSP_ETH, true);
  BSP_LedsInit();

  // Reset external EMAC/PHY
  reg  = BSP_RegisterRead (&BC_REGISTER->CLKRST);
  reg |= (1 << BC_CLKRST_ETH_SHIFT);
  BSP_RegisterWrite (&BC_REGISTER->CLKRST, reg);
  osDelay(5);
  reg &= ~(1 << BC_CLKRST_ETH_SHIFT);
  BSP_RegisterWrite (&BC_REGISTER->CLKRST, reg);
  osDelay(5);

  net_initialize ();

  osThreadCreate (osThread(Display), NULL);

  while (1) {
    net_main ();
    dhcp_check ();
    osThreadYield ();
  }
}
